define(['controllers/AssessmentWeb', 'widgets/Popup', 'widgets/Spinner'], function(AssessmentWeb, Popup, Spinner) {
	var events = {};
	var popup = null;
	AssessmentUI = {
		currentPage: 0,
		dialog: null,
		init: function(args) {
			jQuery.extend(this, args);
			AssessmentEngine.events.timerUpdated.subscribe(AssessmentUI.updateTimer);
			// show first page
			this.page(1);
			if (!events.controls) {
				events.controls = {};
				events.controls.previous = jQuery(".navbar-gray .nav-page-prev")
					.bind("click", jQuery.proxy(this.previous, this));
				events.controls.next = jQuery(".navbar-gray .nav-page-next")
					.bind("click", jQuery.proxy(this.next, this));
				this.mainContainer = jQuery("main");
				events.controls.swiperight = this.mainContainer.bind("swiperight", jQuery.proxy(this.previous, this));
				events.controls.swipeleft = this.mainContainer.bind("swipeleft", jQuery.proxy(this.next, this));
			}
			this.setCurrentPageNumber();
			Spinner.initInstance({
				parentNode: jQuery(".main-container")
			});
			var buttons = jQuery("<div></div>").css("padding-top", 10);
			var submitBtn = jQuery("<button></button>").html('Confirm').addClass('btn btn-success btn-block');
			buttons.append(submitBtn);
			var popup = this.getPopup();
			popup.setWidth(300);
			popup.setValues("Submit answers?", buttons, null);
			submitBtn.on('click', jQuery.proxy(function() {
				// this.hide dialog method is called as "this" is scoped to dialog
				Spinner.show();
				AssessmentEngine.score();
				this.discard();
				Spinner.hide();
			}, popup));
			jQuery(".btn-submit")
				.bind("click", jQuery.proxy(function() {
					if (this.menuWidget && this.menuWidget.visible) {
						this.menuWidget.toggle();
					}
					popup.display();
				}, this));
		},
		getPopup: function(params) {
			if (!popup) {
				this.popup = new Popup({
					container: jQuery(".main-container")
				});
				this.popup.init();
				popup = this.popup;
			}
			return popup;
		},
		getPageCount: function() {
			return jQuery(".page")
				.length;
		},
		page: function(p, pushElements) {
			var elms = jQuery(".page");
			if (!p) {
				var p = parseInt(jQuery("#pgctl")
					.val());
				if (isNaN(p) || p < 1 || p > elms.length) {
					return;
				}
			}
			if (!!pushElements) pushElements(p);
			if (this.menuWidget) {
				if (this.currentPage > 0 && AssessmentEngine.isPageCompleted(this.currentPage)) {
					this.menuWidget.setCheckmark("assessment-page-" + this.currentPage);
				} else {
					this.menuWidget.unsetCheckmark("assessment-page-" + this.currentPage);
				}
				this.menuWidget.setActiveNavElement(null, "assessment-page-" + p);
			}
			this.currentPage = p;
			AssessmentEngine.currentPage = p;
			this.setCurrentPageNumber();
			this.pushPageToCenter(jQuery(elms[this.currentPage - 1]));
		},
		pushPageToLeftAnimated: function(elem) {
			elem.addClass("pushed-left");
		},
		pushPageToLeftNoAnimation: function(elem) {
			elem.removeClass("pushed-right");
			elem.addClass("pushed-left");
		},
		pushPagesToLeft: function(newPageNumber) {
			var elms = jQuery(".page");
			for (var i = 0; i < newPageNumber - 1; i++) {
				// only animate the transition off-screen of the currently visible page
				if (i + 1 !== this.currentPage) {
					this.pushPageToLeftNoAnimation(jQuery(elms[i])); // Move all relevant pages to the left without animation
				} else {
					this.pushPageToLeftAnimated(jQuery(elms[i]));
				}
			}
		},
		pushPageToRightAnimated: function(elem) {
			elem.addClass("pushed-right");
		},
		pushPageToRightNoAnimation: function(elem) {
			elem.removeClass("pushed-left");
			elem.addClass("pushed-right");
		},
		pushPagesToRight: function(newPageNumber) {
			var elms = jQuery(".page");
			for (var i = newPageNumber - 1; i < elms.length; i++) {
				// only animate the transition off-screen of the currently visible page
				if (i + 1 !== this.currentPage) {
					this.pushPageToRightNoAnimation(jQuery(elms[i]));
				} else {
					this.pushPageToRightAnimated(jQuery(elms[i]));
				}
			}
		},
		pushPageToCenter: function(elem) {
			elem.removeClass("pushed-left")
				.removeClass("pushed-right");
		},
		setCurrentPageNumber: function() {
			if (this.currentPage) jQuery(".navbar-gray .navbar-page-number")
				.html(this.currentPage + " of " + jQuery(".page")
					.length);
		},
		previous: function() {
			this.goToPage(this.currentPage - 1);
		},
		next: function() {
			this.goToPage(this.currentPage + 1);
		},
		goToPage: function(pageNumber) {
			var pageCount = this.getPageCount();
			if (pageNumber > 0 && pageNumber <= pageCount && pageNumber !== this.currentPage) {
				if (pageNumber <= this.currentPage) {
					this.page(pageNumber, jQuery.proxy(this.pushPagesToRight, this));
				} else {
					if ((!AssessmentEngine.assessment.allowSkipQuestions && !AssessmentEngine.assessment.allowBackwardsNavigation) && !AssessmentEngine.isPageCompleted(this.currentPage)) {
						// not allowed to navigate backwards and not allowed to skip
						// and the page has unanswered questions
						alert(AssessmentEngine.skippedNotAllowedMsg);
					} else {
						this.page(pageNumber, jQuery.proxy(this.pushPagesToLeft, this));
					}
				}
			}
		},
		updateTimer: function() {
			if (AssessmentEngine.remaining > 0) {
				var elm = jQuery("#tmrval");
				if (elm.length > 0) {
					elm.html(AssessmentUI.formatDisplayTime(AssessmentEngine.remaining));
				}
			}
		},
		formatDisplayTime: function(remainingSeconds) {
			var h = 0,
				m = 0,
				s = 0;
			if (remainingSeconds < 60) {
				s = remainingSeconds;
			} else if (remainingSeconds < 3600) {
				m = Math.floor(remainingSeconds / 60);
				s = (remainingSeconds - (m * 60));
			} else {
				h = Math.floor(remainingSeconds / 3600);
				m = Math.floor((remainingSeconds - (h * 3600)) / 60);
				s = (remainingSeconds - (h * 3600) - (m * 60));
			}
			if (m < 10) m = "0" + m;
			if (s < 10) s = "0" + s;
			return h + ":" + m + ":" + s;
		}
	};
	return AssessmentUI;
});

