define([
 "widgets/SlidingMenu",
 "plugins/text!widgets/templates/Search.html",
 "controllers/SearchController"
], function(SlidingMenu, template, SearchController) {
    var ReturnCode = 13; // when enter/return key pressed
    var BackSpaceCode = 8; // when backspace key pressed
    var _this = null;
    return function(args) {
        var search = {
            domNode: null,
            sideMenu: null,
            searchController: null,
            navigationController: null,
			homeButton: null,
			searchInput: null,
            searchIcon: null,
            closeButton: null,
            inSearch: null,
            searchString: null,
            init: function() {
                jQuery.extend(this, args);
                _this = this;
                var searchTemplate = jQuery(template)
                    .clone();
                this.domNode = searchTemplate.get();
                this.searchController = new SearchController({
                    slidingMenu: this.sideMenu
                });
                this.searchIcon = jQuery(searchTemplate)
                    .find(".search-icon-addon");    
				this.homeButton = jQuery(searchTemplate)
                    .find(".search-addon");

                this.searchInput = jQuery(searchTemplate)
                    .find(".search-input");
					
                this.homeButton.click(function() {
					if ( _this.inSearch == true )
					{
						_this.searchController.searchEnd();
						_this.closeButton.hide();                       
						_this.inSearch = false;
						_this.searchInput.val("");
					}
					_this.searchController.navigateRoot();
                });
					
                this.closeButton = jQuery(searchTemplate)
                    .find(".close-addon");
                this.closeButton.click(function() {
                    _this.searchController.searchEnd();      
                    _this.closeButton.hide();
                    _this.searchIcon.show();   
                    _this.inSearch = false;
					_this.searchInput.val("");
                });
                // Hide the close button to start with. When a user types in something we will show it.
                this.closeButton.hide();
                jQuery(searchTemplate)
                    .find(".search-input")
                    .keypress(function(e) {
                        if (e.which === ReturnCode) {
                            _this.search(this.value);    
                        }
                    });
                jQuery(searchTemplate)
                    .find(".search-input")
                    .keydown(function(e) {
                        if (this.value.length == 1 && e.which == BackSpaceCode && !_this.inSearch) {
                            // If the search was not started, then we hide the close button when the user clears the text.
                            _this.closeButton.hide();
                        } else {

                            _this.closeButton.show();
                            _this.searchIcon.hide();
                            //required by IE to display close icon on the right of search input
                            _this.closeButton.css("display", "table-cell" ); 
                        }
                    });
            },
            setNavigationController: function(navigationController) {
                this.navigationController = navigationController;
            },
            // Todo: Need to use the JSONUtil method instead of copying it here.
            removeTaintedValue: function(jSONValue, taintedValue) {
                var tmp;
                if (taintedValue) tmp = taintedValue;
                else tmp = "for(;;);";
                return jSONValue.replace(tmp, "");
            },
            /* function to perform the search operation */
            search: function(searchString) {
				if (typeof inExport != "undefined" && inExport == true) {
					alert("Search not available for exported courses");
					return;
				}                // Save the search String.
                _this.searchString = searchString;
                jQuery.ajaxSetup({
                    dataFilter: function(jsonString, type) {
                        var result = jsonString;
                        if (type == "json") {
                            result = _this.removeTaintedValue(result);
                        }
                        return result;
                    }
                });
				var objID = localStorage["responsiveViewer.rootCourseId"];
				/* Temporarily Commented out. We will not have contextual Search.
				var currentNodeObject = _this.sideMenu.treeController.currentNodeObject;
				if (currentNodeObject != null)
				{
					objID = currentNodeObject.id;
				}
				*/
				
                var request = jQuery.ajax({
                    url: "AjaxServlet",
                    type: "POST",
                    data: {
                        clsID: 66,
                        query: searchString,
                        agg: 11,
                        objectID: objID
                    },
                    dataType: "json",
                });
                request.success(function(returnValue) {
                    _this.sideMenu.createNavigationList();
                    var resultsCount = returnValue.results.length;
                    if (resultsCount > 0) {
                        jQuery(returnValue.results)
                            .each(function(index, obj) {
                                var callback = function(obj, index) {
                                    return function() {
                                        navigationCallback(obj, index);
                                    }
                                };
                                _this.sideMenu.generateSearchNavItem(obj.label, index, callback(obj, index), false);
                            });
                    } else {
                        var noResultscallback = function() {
                            alert("No results... Please try with a different search criteria.");
                        };
                        _this.sideMenu.generateSearchNavItem("No Results", 0, noResultscallback, false);
                    }
                    _this.sideMenu.displaySearchList();
                    _this.searchController.searchStart(_this.searchString);
                    _this.inSearch = true;
                });
                request.fail(function(jqXHR, textStatus, errorThrown) {
                    alert("Request failed: " + jqXHR.resultText + " " + textStatus + " " + errorThrown);
                });
                var navigationCallback = function(obj, pageNumber) {
                    //var url = ViewerController.urlMap[obj.uniqueId];
                    var localObj = OSDeliveryEngine.getObjectByUniqueId(obj.uniqueId);
					// Can't find the object using uniqueId (because the search context was not complete), try to find it using the objectId
					if (localObj == null )
					{
						var matchingObjects = OSDeliveryEngine.getObject(obj.id);
						localObj = matchingObjects[0];
					}
                    _this.navigationController.navigateTo(localObj);
                    _this.sideMenu.setActiveNavElement(null, "search-page-" + pageNumber);
					_this.searchController.setActiveObject(localObj);
                };
            }
        };
        jQuery(search.init());
        return search;
    };
});

