define(["plugins/text!widgets/templates/SlidingMenu.html",
 "widgets/Overlay",
 "widgets/Search"
 ], function(template, Overlay, Search) {
    return function(args) {
        var slidingmenu = {
            list: null, // currently displayed list
            nextList: null, // off-screen generated list
            searchNode: null,
            hideOnNavigate: false,
            navigationController: null,
            treeController: null,
            contentController: null,
            searchWidget: null,
            inSearch: false,
            searchString: null,
            init: function() {
                jQuery.extend(this, args);
                this.containingNode = jQuery(".main-container");
                var slidingMenuTemplate = jQuery(template)
                    .clone();
                this.domNode = jQuery(slidingMenuTemplate)
                    .get();
                Overlay.initInstance({
                    parentNode: this.containingNode
                });
                //var searchWidget = new Search();
                this.searchWidget = new Search({
                    sideMenu: this
                });
                this.searchNode = this.searchWidget.domNode;
                jQuery(this.domNode)
                    .append(this.searchNode);
                jQuery(Overlay.domNode)
                    .bind("click", jQuery.proxy(function() {
                        if (this.visible && !this.isTablet()) {
                            this.toggle();
                        }
                    }, this));
                jQuery(".nav-button-slideout")
                    .bind("click", jQuery.proxy(this.toggle, this));
                /*
                 * resize/orientation change events close the sidemenu if it is open
                 */
                jQuery(window)
                    .on("resize", jQuery.proxy(function() {
                        if (this.visible) {
                            this.toggle();
                        }
                    }, this));
                if (this.isTablet()) {
                    this.toggle(); // show and set-up initial classes for overlay, etc.
                } else {
                    this.hide(); // set up initial classes to ensure animations run properly, etc.
                }
            },
            setInSearch: function(inSearch, searchString) {
                this.inSearch = inSearch;
                this.searchString = searchString;
                this.contentController.setInSearch(inSearch, searchString);
            },
            isInSearch: function() {
                return this.inSearch;
            },
            setNavigationController: function(navigationController) {
                this.navigationController = navigationController;
                this.searchWidget.setNavigationController(navigationController);
            },
            setTreeController: function(treeController) {
                this.treeController = treeController;
            },
            setContentController: function(contentController) {
                this.contentController = contentController;
            },
            // helper function
            isTablet: function() {
                return window.matchMedia("(min-width: 768px)")
                    .matches;
            },
            // For the parameter, slideDirection: true is right, false is left
            createNavigationList: function(slideDirection) {
                var navList = jQuery("<ul></ul>")
                    .addClass("list-group sidemenu-root");
                if (slideDirection) {
                    navList.addClass("slide-right");
                } else {
                    navList.addClass("slide-left");
                }
                this.nextList = navList;
                jQuery(".sidemenu")
                    .append(this.nextList);
            },
            createBackButton: function(backObject, callback) {
                var listItem = jQuery("<li></li>")
                    .addClass("list-group-item back");
                listItem.attr("id", backObject.id);
                var caretIcon = listItem.append(jQuery("<span class='caret'></span>"));
                listItem.append(backObject.label);
                listItem.bind("click", callback);
                jQuery(this.nextList)
                    .append(listItem);
            },
            displayNavigationList: function() {
                jQuery(this.list)
                    .remove(); // remove the previous list
                /*
                 * forces the newly created list items to be laid out before the
                 * CSS animation Fixes a nasty race condition. (see
                 * http://stackoverflow.com/questions/12088819/css-transitions-on-new-elements)
                 */
                this.nextList.children("li")[0].clientHeight;
                if (jQuery(this.nextList)
                    .hasClass("slide-left")) {
                    jQuery(this.nextList)
                        .removeClass("slide-left");
                } else {
                    jQuery(this.nextList)
                        .removeClass("slide-right");
                }
                this.list = this.nextList;
                /*
                 * determine if the new navigation list is at the top level if
                 * it is show the search bar, otherwise hide it
                 */
                var children = this.nextList.children("li");
                var nodes = [];
                for (var i = 0; i < children.length; i++) {
                    var childObjects = OSDeliveryEngine.getObject(children[i].id);
                    if (childObjects && childObjects.length > 0) {
                        nodes.push(childObjects[0]);
                    }
                }
                var result = jQuery.grep(nodes, function(e) {
                    if (e.parent) {
                        return e.parent.id === ViewerController.root.id;
                    }
                    return false;
                });
                // if only one entry, then no back button, must show search else
                // if more than one entry and more than one has parent id ===
                // root id, then show
				/* DPV - 10/23 Always show the search node now at all level.
                if (children.length === 1 || result.length > 1) {
                    jQuery(this.searchNode)
                        .show();
                } else {
                    jQuery(this.searchNode)
                        .hide();
                }
				*/
            },
            displaySearchList: function() {
                jQuery(this.list)
                    .remove(); // remove the previous list
                /*
                 * forces the newly created list items to be laid out before the
                 * CSS animation Fixes a nasty race condition. (see
                 * http://stackoverflow.com/questions/12088819/css-transitions-on-new-elements)
                 */
                this.nextList.children("li")[0].clientHeight;
                if (jQuery(this.nextList)
                    .hasClass("slide-left")) {
                    jQuery(this.nextList)
                        .removeClass("slide-left");
                } else {
                    jQuery(this.nextList)
                        .removeClass("slide-right");
                }
                this.list = this.nextList;
                /*
                 * determine if the new navigation list is at the top level if
                 * it is show the search bar, otherwise hide it
                 */
                var children = this.nextList.children("li");
                var nodes = [];
                for (var i = 0; i < children.length; i++) {
                    var childObjects = OSDeliveryEngine.getObject(children[i].id);
                    if (childObjects && childObjects.length > 0) {
                        nodes.push(childObjects[0]);
                    }
                }
            },
            toggle: function() {
                if (this.visible) {
                    Overlay.hide();
                    this.hide();
                } else {
                    Overlay.showWithAnimation();
                    this.show();
                }
                /* 
                 * Event to inform listeners that the menu has been toggled
                 * Using this for components that need to resize based on
                 * activity from the sidemenu.
                 */
                jQuery(window)
                    .trigger(jQuery.Event("sideMenuToggled"));
            },
            show: function() {
                this.containingNode.addClass("sidemenu-show-left");
                this.containingNode.removeClass("sidemenu-hide-left"); // for tablets
                this.visible = true;
                this.onShow();
            },
            hide: function() {
                this.containingNode.removeClass("sidemenu-show-left");
                this.containingNode.addClass("sidemenu-hide-left"); // for tablets
                this.visible = false;
                this.onHide();
            },
            navCallback: function(originalCallback) {
                if (this.hideOnNavigate) {
                    if (!this.isTablet()) {
                        this.toggle();
                    }
                }
                originalCallback();
            },
            onShow: function() {},
            onHide: function() {},
            isVisible: function() {
                return this.visible;
            },
            generateSearchNavItem: function(label, pageNumber, callback, showCheckmark) {
                var listItem = jQuery("<li></li>")
                    .addClass("list-group-item navigation-depth-1");
                listItem.attr("id", "search-page-" + pageNumber);
                var checkmark = jQuery("<span></span>")
                    .addClass("checkmark");
                listItem.append(checkmark);
				/*
				if ( !ViewerController.isIdentified(obj) ) {
					showCheckmark = false;
				}
				else if ( ViewerController.isIdentifiedByAssessment(obj) ) {
					// If the group items are identified by assessment, then we need to add some style so that they show differently.
					listItem.addClass("assessment-identified");
				}
				*/
				
                if (!showCheckmark) {
                    checkmark.hide();
                }
				
                listItem.append(label);
                listItem.bind("click", jQuery.proxy(this.navCallback, this, callback));
                this.nextList.append(listItem);
            },
            generateAssessmentNavItem: function(label, pageNumber, callback, showCheckmark) {
                var listItem = jQuery("<li></li>")
                    .addClass("list-group-item navigation-depth-1");
                listItem.attr("id", "assessment-page-" + pageNumber);
                var checkmark = jQuery("<span></span>")
                    .addClass("checkmark");
                listItem.append(checkmark);
				if (!showCheckmark) {
                    checkmark.hide();
                }
                listItem.append(label);
                listItem.bind("click", jQuery.proxy(this.navCallback, this, callback));
                this.nextList.append(listItem);
            },
            generateNavItem: function(obj, titleClickedCallback, iconClickedCallback, parent, isLeaf, showCheckmark) {
                var listItem = jQuery("<li></li>")
                    .addClass("list-group-item navigation-depth-1");
                listItem.attr("id", obj.id);
                var checkmark = jQuery("<span></span>")
                    .addClass("checkmark");
                listItem.append(checkmark);
				if ( !ViewerController.isIdentified(obj) ) {
					showCheckmark = false;
				}
				else if ( ViewerController.isIdentifiedByAssessment(obj) ) {
					// If the group items are identified by assessment, then we need to add some style so that they show differently.
					listItem.addClass("assessment-identified");
				}
				if (!showCheckmark) {
                    checkmark.hide();
                }
                if (!isLeaf) {
                    var caretIcon = listItem.append(jQuery("<span class='caret'></span>"));
                }
                listItem.append(obj.label);
                listItem.bind("click", jQuery.proxy(function(e) {
                    if (!jQuery(e.target)
                        .hasClass("caret")) {
                        this.navCallback(titleClickedCallback);
                    } else {
                        iconClickedCallback();
                    }
                }, this));
                if (parent) {
                    parent.append(listItem);
                } else {
                    jQuery(this.nextList)
                        .append(listItem);
                }
                return listItem;
            },
            /* set the styling for the active navigation element */
            setActiveNavElement: function(elementClicked, activeElementId) {
                /* set the correct item as active */
                activeNavElement = jQuery("#" + activeElementId);
                if (activeNavElement) {
                    jQuery(".sidemenu")
                        .find("*")
                        .removeClass("active");
                    activeNavElement.addClass("active");
                }
            },
            setBreadcrumbs: function(arrayOfIds) {
                jQuery(this.nextList)
                    .children()
                    .removeClass("sidemenu-breadcrumb");
                /*
                 * go through each list item checking ids
                 * 
                 * if id matches in arrayOfIds, set class and return
                 * 
                 */
                var result = jQuery.grep(this.nextList.children(), function(e) {
                    return arrayOfIds.indexOf(e.id) !== -1;
                });
                jQuery(result)
                    .addClass("sidemenu-breadcrumb");
            },
            // shows the checkmark on the specified list item
            setCheckmark: function(listItemOrId) {
                if (jQuery.type(listItemOrId) !== 'object') {
                    jQuery("#" + listItemOrId)
                        .children(".checkmark")
                        .show();
                } else {
                    listItemOrId.children(".checkmark")
                        .show();
                }
            },
            unsetCheckmark: function(listItemOrId) {
                if (jQuery.type(listItemOrId) !== 'object') {
                    jQuery("#" + listItemOrId)
                        .children(".checkmark")
                        .hide();
                } else {
                    listItemOrId.children(".checkmark")
                        .hide();
                }
            }
        };
        jQuery(slidingmenu.init());
        return slidingmenu;
    };
});

